-- tester for fastevent counter.

library ieee;
use ieee.std_logic_1164.all;
use ieee.math_real.all;
use ieee.numeric_std.all;

entity fastevent_tb is end;

architecture test of fastevent_tb is

  -- Minimum pulse width.
  -- This is the shortest pulse that can be detected by a flipflop
  -- Need to get real figure from device/library data sheet; look for
  -- the minimum clock pulse width. NOT the same as maximum clock frequency.
  -- About a nanosecond is probably in the ballpark.
  constant width : time := 1 ns;

  signal trig,
        enable : std_logic := '0';


  signal reset : std_logic := '0';
  signal clk : std_logic;

  signal count : std_logic_vector(7 downto 0);

  signal stop : boolean;

begin

  fastevent_counter_i :
    entity work.fastevent_counter
      port map (sysclk   => clk,
                rstcount => reset,
                enable   => enable,
                trig     => trig,
                count    => count);

clkgen : process begin
  while not stop loop
    clk <= '0';
    wait for 10 ns;
    clk <= '1';
    wait for 10 ns;
  end loop;
  wait;
end process;


-- process to generate some fast pulses for counter.
do_something : process

  variable num : natural;

  variable s1, s2: positive := 42;

  impure function dist_uniform(min, max: integer) return integer is
    variable r: real;
  begin
    uniform(s1, s2, r);
    return min + integer(trunc(real(max+1-min) * r));
  end;

  procedure pulse (num : natural;
                   signal sig : out std_logic) is
    variable gap : time;
  begin
    for I in 1 to num loop
      sig <= '1';
      wait for width;
      sig <='0';
      gap := 1 ns * dist_uniform(100, 200);
      wait for gap;
    end loop;
  end procedure;


begin
  wait for 10 ns;
  enable <= '1';
  wait until falling_edge(clk);
  reset <= '1';
  wait until falling_edge(clk);
  reset <= '0';
  wait until falling_edge(clk);
  wait until rising_edge(clk);
  num := dist_uniform(5, 10);
  pulse(num, trig);
  wait until falling_edge(clk);
  assert (to_integer(unsigned(count)) = num) report "broken";
  reset <= '1';
  wait until falling_edge(clk);
  reset <= '0';
  wait until falling_edge(clk);
  num := dist_uniform(5, 10);
  pulse(num, trig);
  wait until falling_edge(clk);
  assert (to_integer(unsigned(count)) = num) report "broken";
  pulse (1, trig);
  wait until falling_edge(clk);
  stop <= true;
  wait;
end process;

end architecture;